export class Main extends Indicator {

    constructor() {
        super();

        this.setProperty(Property.SeparateWindow, true);

        this.addInput("NBars", InputType.int, 20);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            MNKBuffer : this.addBuffer(),
            MNKBuffer1 : this.addBuffer(),
            MNKBuffer2 : this.addBuffer(),
            MNKBuffer3 : this.addBuffer(),
            MNKBuffer4 : this.addBuffer(),
            MNKBuffer5 : this.addBuffer(),
            MNKBuffer6 : this.addBuffer(),
            MNKBuffer7 : this.addBuffer()

        };
    }

    onInit() {
        let {NBars} = this.getInputs(),
            shortName = "Extremum(" + NBars + ")";

        this.buffers.MNKBuffer
            .setShape(Shape.Line)
            .setColor(Color.Yellow)
            .setDrawBegin(NBars)
            .setLabel("MNK")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;

        this.buffers.MNKBuffer1
            .setShape(Shape.Line)
            .setColor(Color.Lime)
            .setDrawBegin(NBars)
            .setLabel("MNK1")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;

        this.buffers.MNKBuffer2
            .setShape(Shape.Histogram)
            .setColor(Color.Lime)
            .setDrawBegin(NBars)
            .setLabel("MNK2")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;

        this.buffers.MNKBuffer3
            .setShape(Shape.Histogram)
            .setColor(Color.Lime)
            .setDrawBegin(NBars)
            .setLabel("MNK3")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;

        this.buffers.MNKBuffer4
            .setShape(Shape.Line)
            .setColor(Color.Red)
            .setDrawBegin(NBars)
            .setLabel("MNK4")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;

        this.buffers.MNKBuffer5
            .setShape(Shape.Histogram)
            .setColor(Color.Yellow)
            .setDrawBegin(NBars)
            .setLabel("MNK5")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;

        this.buffers.MNKBuffer6
            .setShape(Shape.Histogram)
            .setColor(Color.Red)
            .setDrawBegin(NBars)
            .setLabel("MNK6")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;

        this.buffers.MNKBuffer7
            .setShape(Shape.Histogram)
            .setColor(Color.Magenta)
            .setDrawBegin(NBars)
            .setLabel("MNK7")
            .setDigits(Current.Digits+4)
            .setShortName(shortName)
        ;
    }

    async onUpdate() {
        const {MNKBuffer, MNKBuffer1, MNKBuffer2, MNKBuffer4, MNKBuffer5, MNKBuffer7} = this.buffers,
            {NBars, PriceType} = this.getInputs(),
            {High, Low} = await Bar.load([Bar.Mode.High, Bar.Mode.Low], PriceType),
            barSize = Low.length;

        let limit, counted_bars = Bar.counted(), k, i, n, m, h, hi;

        if (counted_bars > 0) counted_bars--;
        limit = barSize - counted_bars;
        for (k = 0; k < limit; k++) {
            n = 0;
            for (i = k; i < NBars + k; i++) {
                h = High[k];
                hi = High[i];
                if (h > hi && h - hi > n) {
                    n = h - hi;
                }
            }
            m = 0;
            for (i = k; i < NBars + k; i++) {
                if (Low[k] < Low[i] && Low[k] - Low[i] < m) {
                    m = Low[k] - Low[i];
                }
            }
            MNKBuffer.set(k, n + m);
            MNKBuffer1.set(k, m - n);
            MNKBuffer4.set(k, n - m);
            if ((n + m) === (m - n)) {
                MNKBuffer2.set(k, -(n + m) / 2);
            }
            if ((n + m) === (n - m)) {
                MNKBuffer2.set(k, -(n + m) / 2);
            }
            if ((n + m) < 0) {
                MNKBuffer5.set(k,  n + m);
            }
            if ((n + m) > 0) {
                MNKBuffer7.set(k, n + m);
            }
        }

    }
}
